package com.ruoyi.web.controller.system;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.RedisConstant;
import com.ruoyi.common.constant.WebsocketConst;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.framework.websocket.WebSocket;
import com.ruoyi.system.domain.NoticeUserSelect;
import com.ruoyi.system.domain.SysNoticeUser;
import com.ruoyi.system.domain.SysUserOnline;
import com.ruoyi.web.utils.SendNotice.NoticeUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.SysNotice;
import com.ruoyi.system.service.ISysNoticeService;

import javax.annotation.PostConstruct;
import javax.websocket.Session;

/**
 * 公告 信息操作处理
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/notice")
public class SysNoticeController extends BaseController
{
    @Autowired
    private ISysNoticeService noticeService;

    /**
     * 获取通知公告列表
     */
    @PreAuthorize("@ss.hasPermi('system:notice:list')")
    @GetMapping("/list")
    public TableDataInfo list(SysNotice notice)
    {
        startPage();
        List<SysNotice> list = noticeService.selectNoticeList(notice);
        return getDataTable(list);
    }

    /**
     * 根据通知公告编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:notice:query')")
    @GetMapping(value = "/{noticeId}")
    public AjaxResult getInfo(@PathVariable Long noticeId)
    {
        return success(noticeService.selectNoticeById(noticeId));
    }

    /**
     * 新增通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:add')")
    @Log(title = "通知公告", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysNotice notice)
    {
        try {
            notice.setCreateBy(getUsername());
            noticeService.insertNotice(notice);
            //推送消息插入中间表
            LoginUser loginUser = getLoginUser();//当前登陆者
            NoticeUtil.sendNotices(loginUser,notice,noticeService,null);
        }catch (Exception e){
            return error("系统异常");
        }
        return success();
    }

    /**
     * 修改通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:edit')")
    @Log(title = "通知公告", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysNotice notice)
    {
        notice.setUpdateBy(getUsername());
        return toAjax(noticeService.updateNotice(notice));
    }

    /**
     * 删除通知公告
     */
    @PreAuthorize("@ss.hasPermi('system:notice:remove')")
    @Log(title = "通知公告", businessType = BusinessType.DELETE)
    @DeleteMapping("/{noticeIds}")
    public AjaxResult remove(@PathVariable Long[] noticeIds)
    {
        return toAjax(noticeService.deleteNoticeByIds(noticeIds));
    }

    /**
     * 导航面板 消息通知
     * @return
     */
    @GetMapping("/navbarNoticelist")
    public Map<String, List<SysNoticeUser>> navbarNoticelist()
    {
        String userName = getLoginUser().getUsername();//当前登陆者
        List<SysNoticeUser> list = noticeService.navbarNoticelist(userName);
        Map<String, List<SysNoticeUser>> groupedByNoticeType = list.stream()
                .collect(Collectors.groupingBy(SysNoticeUser::getNoticeType));
        for(String key:groupedByNoticeType.keySet()){
            List<SysNoticeUser> lt =  groupedByNoticeType.get(key);
            lt.sort((t1, t2) -> t2.getSendTime().compareTo(t1.getSendTime()));
        }
        return groupedByNoticeType;
    }

    /**
     * 导航面板 消息通知-更多
     * @return
     */
    @GetMapping("/navbarNoticeMorelist")
    public List<Map<String, Object>> navbarNoticeMorelist(SysNoticeUser sysNoticeUser)
    {
        String userName = getLoginUser().getUsername();//当前登陆者
        String noticeType = sysNoticeUser.getNoticeType();//消息类型
        sysNoticeUser.setCurrentUserName(userName);
        sysNoticeUser.setNoticeType(noticeType);
        List<SysNoticeUser> list = noticeService.navbarNoticeMorelist(sysNoticeUser);

        Map<String, Object> map = new HashMap<>();
        List<Map<String, Object>> listMap = new ArrayList<Map<String, Object>>();

        Map<String, List<SysNoticeUser>> groupedByNoticeType = list.stream()
                .collect(Collectors.groupingBy(SysNoticeUser::getSendTimeFormat));
        for(String key:groupedByNoticeType.keySet()){
            List<SysNoticeUser> lt =  groupedByNoticeType.get(key);
            lt.sort((t1, t2) -> t2.getSendTime().compareTo(t1.getSendTime()));
            map = new HashMap<>();
            map.put("date",key);
            map.put("data",lt);
            listMap.add(map);
        }
        return listMap;
    }

    /**
     * 导航面板 消息通知 获取详细信息
     */
    @GetMapping(value = "navbarNotice/{noticeId}")
    public AjaxResult getNavbarNoticeInfo(@PathVariable Long noticeId)
    {
        try{
            String userName = getLoginUser().getUsername();//当前登陆者
            //更新sys_user_notice推送消息插入中间表 isRead字段 设置 为 1-已读
            SysNoticeUser sysNoticeUser = new SysNoticeUser();
            sysNoticeUser.setNoticeId(noticeId);
            sysNoticeUser.setUserId(userName);
            sysNoticeUser.setIsRead("1");
            noticeService.updateSysNoticeUser(sysNoticeUser);

        }catch(Exception e){
            return error("系统异常");
        }
        return success(noticeService.selectNoticeById(noticeId));
    }

    /**
     * 获取对应消息对象的 人员
     * @param noticeUserSelect
     * @return
     */
    @GetMapping("/listNoticeEventUser")
    public List<NoticeUserSelect> listNoticeEventUser(NoticeUserSelect noticeUserSelect)
    {
        List<NoticeUserSelect> list = noticeService.listNoticeEventUser(noticeUserSelect);
        return list;
    }

    @PostMapping("/saveNoticeEventUser")
    public AjaxResult saveNoticeEventUser(@RequestBody List<NoticeUserSelect> list)
    {
        try{
            if(list!=null&&list.size()>0){
                String noticeEventType = list.get(0).getNoticeEventType();
                noticeService.deleteNoticeEventUserByType(noticeEventType);
                noticeService.insertNoticeEventUserBatch(list);
            }
        }catch (Exception e){
            return error("系统异常");
        }
        return success();
    }

    @PostMapping("/delNoticeEventUser")
    public AjaxResult delNoticeEventUser(@RequestBody NoticeUserSelect noticeUserSelect)
    {
        try{
            noticeService.deleteNoticeEventUser(noticeUserSelect);
        }catch (Exception e){
            return error("系统异常");
        }
        return success();
    }
}
