package com.ruoyi.web.utils.SendNotice;

import com.alibaba.fastjson2.JSONObject;
import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.WebsocketConst;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.websocket.redisWebsocket.WebsocketService;
import com.ruoyi.system.domain.SysNotice;
import com.ruoyi.system.domain.SysNoticeUser;
import com.ruoyi.system.service.ISysNoticeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

public class NoticeUtil {

    @Autowired
    private static RedisCache redis = SpringUtils.getBean(RedisCache.class);

    @Autowired
    private static WebsocketService wbService = SpringUtils.getBean(WebsocketService.class);

    /**
     * 推送消息插入中间表
     * @param loginUser 发送对象排除自身
     * @param notice 发送信息对象
     * @param noticeService 接口
     */
    public static void sendNotices(LoginUser loginUser, SysNotice notice, ISysNoticeService noticeService, List<String> userIds) throws IOException {
        JSONObject obj = new JSONObject();
        obj.put(WebsocketConst.MSG_ID, notice.getNoticeId());
        obj.put(WebsocketConst.MSG_TITLE, notice.getNoticeTitle());
        obj.put(WebsocketConst.MSG_CONTENT, notice.getNoticeContent());

        if(!"3".equals(notice.getNoticeType())){//系统通知 或 通知公告

            wbService.sendMessageAll("ruoyi",obj.toString());

            insertNoticeUser(loginUser,notice,noticeService,null);

        }else{
            if(userIds!=null&&userIds.size()>0){
                for(String userId:userIds){
                    wbService.sendMessageById("ruoyi",userId,obj.toString());
                }

                insertNoticeUser(loginUser,notice,noticeService,userIds);

            }
        }
    }

    public static void insertNoticeUser(LoginUser loginUser,SysNotice notice,ISysNoticeService noticeService, List<String> userIds){
        List<SysNoticeUser> sysNoticeUsers = new ArrayList<SysNoticeUser>();
        //发送日期
        Date sendTime = DateUtils.getNowDate();
        //获取在线用户
        Collection<String> keys = redis.keys(CacheConstants.LOGIN_TOKEN_KEY + "*");

        if(userIds!=null&&userIds.size()>0){
            for (String key : keys) {
                LoginUser user = redis.getCacheObject(key);
                String userId = user.getUsername();
                if(userIds.contains(userId)){
                    SysNoticeUser sysnoticeuser = noticeSet(loginUser,notice,userId,sendTime);
                    sysNoticeUsers.add(sysnoticeuser);
                }
            }
            if(!sysNoticeUsers.isEmpty()){
                noticeService.insertNoticeUserBatch(sysNoticeUsers);
            }
        }else{
            for (String key : keys) {
                LoginUser user = redis.getCacheObject(key);
                String userId = user.getUsername();
                if(!userId.equals(loginUser.getUsername())){
                    SysNoticeUser sysnoticeuser = noticeSet(loginUser,notice,userId,sendTime);
                    sysNoticeUsers.add(sysnoticeuser);
                }
            }
            if(!sysNoticeUsers.isEmpty()){
                noticeService.insertNoticeUserBatch(sysNoticeUsers);
            }
        }
    }

    /**
     * 消息用户中间表设置
     * @param loginUser 当前登录者
     * @param notice 消息实体类
     * @param userId 在线用户账号
     * @param sendTime 发送时间
     * @return
     */
    public static SysNoticeUser noticeSet(LoginUser loginUser,SysNotice notice,String userId,Date sendTime){
        SysNoticeUser sysNoticeUser  = new SysNoticeUser();
        sysNoticeUser.setNoticeId(notice.getNoticeId());
        sysNoticeUser.setUserId(userId);
        sysNoticeUser.setSendUser(loginUser.getUser().getNickName());
        sysNoticeUser.setSendTime(sendTime);
        return sysNoticeUser;
    }
}
